<?php

class vDrive extends secureInputStream {

   var $drive;
   var $baseDir = "";
   var $mode = 0;
   var $files;

   function vDrive(/* String */ $filename, /* Hashtable */ $OPTIONS, /* String */ $baseDir, /* int */ $mode) {

     settype($filename, "string");
     settype($baseDir, "string");
     settype($mode, "integer");

     if (is_array($OPTIONS)) { $this->attributes = $OPTIONS; }
     if (!file_exists($filename) || !is_readable($filename)) {
       unset($this->source);
       $this->exists = false;
     } else {
       $this->exists = true;
     }
     global $REMOTE_ADDR; if (isset($REMOTE_ADDR)) { $this->attributes[REMOTE_ADDR] = $REMOTE_ADDR; }
     $this->filename = $filename;
     $this->baseDir  = $baseDir;
     $this->mode     = $mode;
   }

   /* public boolean */ function addDrive(/* String */ $filename, /* String */ $baseDir) {
     settype($filename, "string");
     settype($baseDir, "string");

     if (!is_readable($filename)) {
       return false;
     } else {
       $REF = getConfigFile($filename,false);
       if (isset($REF[SETTINGS]) && is_array($REF[SETTINGS])) { $this->options = $REF[SETTINGS]; };
       $this->add_mountpoint($REF, $baseDir);
       return true;
     }
   }

   function mount(/* String */ $key) {
    settype($key, "string");

     if (isset($this->drive[$key])) {
       $return =  $this->drive[$key]->mount();
       if (is_array($return)) {
         $this->options = array_merge_recursive($this->options, $return);
       } else {
         settype($return, "boolean");
         return $return;
       }
     } else {
       return false;
     }
   }

   function read() {
     if (!is_readable($this->filename)) {
       $this->exists  = false;
       return false;
     } else {
       $this->options = $this->attributes;
       $this->content = $this->source = getConfigFile($this->filename,false);
       $this->exists  = true;
       if (isset($this->content[SETTINGS]) && is_array($this->content[SETTINGS])) { $this->options = array_merge_recursive($this->options, $this->content[SETTINGS]); };
       $this->add_mountpoint($this->content, $this->baseDir);
       return true;
     }
   }

   function toString() {
     if ($this->drive) {
       $txt = "";
       foreach ($this->drive as $key=>$element) {
         $txt .= "Mountpoint \"$key\":"."\n\t- source = ".$element->mountpoint->filename."\n\t- type = ".get_class($element->mountpoint)."\n\t- is mounted = ".(($element->mountpoint->isEmpty()) ? 'no' : 'yes')."\n\n";
       }
       return $txt;
     } else {
       return "unmounted virtual Drive\n";
     }
   }

/**********************************************************************************************
* Hilfsfunktionen
**********************************************************************************************/

  function get(/* String */ $key) {
    settype($key, "string");

     if (isset($this->drive[$key]->mountpoint)) {
       return $this->drive[$key]->mountpoint;
     } else {
       return false;
     }
  }

  function add_mountpoint(/* String */ $drive, /* String */ $baseDir) {
    settype($drive, "array");
    settype($baseDir, "string");
    uasort($drive, sortByPriority);

    foreach ($drive as $key => $element) {
      settype($element[SRC], "array");
      if ($this->mode == 1 && isset($element["DEFAULT"])) { $element[SRC] = array($element["DEFAULT"]); };
      if ($element[TYPE]=="dir") { $this->add_mountpoint($element[CONTENT], $baseDir.$element[SRC][0]); };
      for ($i=0; $i<count($element[SRC]); $i++) {
        $element[SRC][$i] = layout::fillTemplate($baseDir.$element[SRC][$i], $this->options, false);
      };
      if (isset($element[TYPE]) && $element[TYPE]=="dir") { $this->drive[$key] = new vDrive_dir($element[SRC],$element[TYPE],($element[AUTOMOUNT]=="yes") ? true : false,$element[FILTER]); };
      if (isset($element[TYPE]) && $element[TYPE]!="dir") { $this->drive[$key] = new vDrive_file($element[SRC],$element[TYPE],($element[AUTOMOUNT]=="yes") ? true : false); };
      if (isset($this->drive[$key]->mountpoint)) { $this->files[$key] = $this->drive[$key]->mountpoint; }
      if ($this->drive[$key]->mountpoint->value) { $this->options     = array_merge_recursive($this->options, $this->drive[$key]->mountpoint->value); }
    };
  }

  function checkDrive() {
     if ($this->isEmpty()) {
        return "Cannot perform check! Drive is not mounted.\n";
     } else {
        $drive[0] = $this->content;
        $i = 0;
        $j = 1;
        $txt = "";

        while ($i<$j) {
            foreach ($drive[$i] as $key => $element) {
              if ($element[TYPE]=="dir" && isset($element[CONTENT])) {$drive[count($drive)] = $element[CONTENT]; };
              $filename = $this->drive[$key]->mountpoint->filename;
              if ($element[SRC]) {
                switch ($element[REQUIRED]) {
                  case "exists":
                    if (!file_exists($filename)) {
                      $txt .= "<P>ERROR: File \"$filename\" is a required ressource, but it does not exist.</P>\n";
                    }                  break;
                  case "readable":
                    if (!is_readable($filename)) {
                      $txt .= "<P>ERROR: File \"$filename\" is not readable.<BR>Please make sure the file exists AND this program has permission to open it.</P>\n";
                    } else {
                      $txt .= checkFilePermission($filename, 444);
                    }
                  break;
                  case "writeable":
                    if (!is_writeable($filename)) {
                      $txt .= "<P>ERROR: File \"$filename\" is not writeable.<BR>Please make sure the file exists AND this program has permission to write at the file and the containing directory.</P>\n";
                    } else {
                      $txt .= checkFilePermission($filename, 666);
                    }                  break;
                  case "create":
                    if (file_exists($filename) && !is_writeable($filename)) {
                      $txt .= "<P>ERROR: File \"$filename\" exists but is not writeable.<BR>Please make sure the file exists AND this program has permission to write at the file and the containing directory.</P>\n";
                    } elseif (file_exists($filename)) {
                      $txt .= checkFilePermission($filename, 666);
                    }                  break;
                  default:
                  break;
                }
              }
            } # FOR
            clearstatcache();
            $i++;
            $j = count($drive);
        } # WHILE

        return $txt;

     } # IF
  }

}

function checkFilePermission($filename, $permission2) {
  $txt = "";
  $permission1 = decoct(fileperms($filename));

  settype($permission1, "string");
  settype($permission2, "string");
  $permission1 = preg_replace("/.*(\d\d\d)$/s", "$1", $permission1);
  $permission2 = preg_replace("/.*(\d\d\d)$/s", "$1", $permission2);

  if (strlen($permission1)>2) {
    $permission[0] = (int) preg_replace("/.*(\d)\d\d$/s", "$1", $permission1);
    $permission[1] = (int) preg_replace("/.*\d(\d)\d$/s", "$1", $permission1);
    $permission[2] = (int) preg_replace("/.*\d\d(\d)$/s", "$1", $permission1);
  }
  if (strlen($permission2)>2) {
    $permission[3] = (int) preg_replace("/.*(\d)\d\d$/s", "$1", $permission2);
    $permission[4] = (int) preg_replace("/.*\d(\d)\d$/s", "$1", $permission2);
    $permission[5] = (int) preg_replace("/.*\d\d(\d)$/s", "$1", $permission2);
  }
  if ($permission[0] < $permission[3] || $permission[1] < $permission[4] || $permission[2] < $permission[5]) {
    $txt .= "<P>ERROR: This script requires a higher permission level for File \"$filename\". <BR> Permissions are actually set to \"$permission1\", required level is \"$permission2\". Use CHMOD command to change permission level. See script documentation for details.</P>\n";
  }

  clearstatcache();
  
  return $txt;
}

function sortByPriority($a,$b) {
  if (!isset($a[PRIORITY])) { $a[PRIORITY] = 0; }
  if (!isset($b[PRIORITY])) { $b[PRIORITY] = 0; }
  if ($a[PRIORITY] == $b[PRIORITY]) { return 0; }
  else { return ($a[PRIORITY] < $b[PRIORITY]) ? 1 : -1; }
}

//  2004, Thomas Meyer www.All-Community.de
// Copyright Hinweis nicht entfernen!
?>
